library(ape)
library(diversitree)
library(phytools)
library(geiger)
library(readxl)
library(picante)

#######################################################
#######################################################
####### MuSSE: epiphytic/terrestrial vs. spore type

##data spore type and habit d (epiphytes (1), terrestrial (0))
musse_epiter<-read_excel("traits_final.xlsx")
musse_epiter<-as.data.frame(musse_epiter)
musse_epiter<-musse_epiter[,c(1,2,6)]
row.names(musse_epiter)<-musse_epiter$Species 
musse_epiter<-na.omit(musse_epiter)


#phylogeny
tree<-read.tree("5Feb_dated_tree.tre",tree.names = NULL)

#prune tree
matchmuet<-match.phylo.data(tree,musse_epiter)
musse_epiter2<-matchmuet$data
treemuet<-matchmuet$phy
treeaultra2 <- force.ultrametric(treemuet)


# Prepare states
states <- paste0(musse_epiter2$spore_type,musse_epiter2$habit_d)
states <- as.factor(states)
states <- as.numeric(states)
names(states) <- musse_epiter2$Species


#sampling factor

#terrestrial achlorophyllous=7742
#epiphyte achlorophyllous=1339
#terrestrial chlorophyllous=231
#epiphyte chlorophyllous=1079


table(states)
#1   2   3   4 
#951 433  46 296 

fet<-c(0.12,0.3,0.2,0.3) 

# musse model
liket <- make.musse(treeaultra2,states, k=4,sampling.f = fet)

# likelihood function
lik.et<-constrain(liket, mu2 ~ mu1, mu3 ~ mu1, mu4 ~ mu1,
                  q14 ~ 0, q41 ~ 0, q23 ~ 0, q32 ~ 0)
p<-starting.point.musse(treeaultra2,k=4)
p

fit.et <- find.mle(lik.et, p[argnames(lik.et)],method = "optim",
                   control=list(optim.method="BFGS"))


#highest rate of maximum likelihood to be used as prior
fit.et$par.full
#> 0.2624152*2
#[1] 0.5248304


#prior

prioret <- make.prior.exponential(0.52)

#preliminar mcmc 
prelimet <- diversitree::mcmc(lik.et, fit.et$par, nsteps=100, prior=prioret, 
                              w=1, lower=0, print.every=0)
summary(prelimet)

coef(fit.et)
fit.et$par
wet <- diff(sapply(prelimet[2:(ncol(prelimet)-1)], quantile, c(0.05, 0.95)))
wet

#mcmc
sampleset1<- diversitree::mcmc(lik.et, fit.et$par, nsteps=10000, w=wet, lower=0, 
                                   prior=prioret, print.every=100, 
                                   save.file="mcmc_chain_epiter1.csv", save.every=5)

sampleset2<- diversitree::mcmc(lik.et, fit.et$par, nsteps=10000, w=wet, lower=0, 
                               prior=prioret, print.every=100, 
                               save.file="mcmc_chain_epiter2.csv", save.every=5)
sampleset3<- diversitree::mcmc(lik.et, fit.et$par, nsteps=10000, w=wet, lower=0, 
                               prior=prioret, print.every=100, 
                               save.file="mcmc_chain_epiter3.csv", save.every=5)
sampleset4<- diversitree::mcmc(lik.et, fit.et$par, nsteps=10000, w=wet, lower=0, 
                               prior=prioret, print.every=100, 
                               save.file="mcmc_chain_epiter4.csv", save.every=5)



# Analyze runs
library(coda)
runet1 <- coda::mcmc(sampleset1[,-1], start=1, thin=1)
runet2 <- coda::mcmc(sampleset2[,-1], start=1, thin=1)
runet3 <- coda::mcmc(sampleset3[,-1], start=1, thin=1)
runet4 <- coda::mcmc(sampleset4[,-1], start=1, thin=1)

runs_et <- coda::mcmc.list(runet1,runet2)

par(mar=rep(2,4))
plot(runs_et)
effectiveSize(runs_et)
View(runs_et)
gelman.plot(runs_et)
gelman.diag(runs_et)

####posterior probabilitiies
et1<-read.csv("mcmc_chain_epiter1___.csv")
et2<-read.csv("mcmc_chain_epiter2___.csv")
et3<-read.csv("mcmc_chain_epiter3___.csv")
et4<-read.csv("mcmc_chain_epiter4___.csv")

#burn in
bi_et1<-subset(et1, i > 1000)
bi_et2<-subset(et2, i > 1000)
bi_et3<-subset(et3, i > 1000)
bi_et4<-subset(et4, i > 1000)

#merge
ppet<-rbind(bi_et1,bi_et2,bi_et3,bi_et4)
summary(ppet)

#Posterior probabilities
mean(ppet$q24 > ppet$q42)
#0.9990278

# 95% HPD rates 
quantile(ppet$mu1,probs=c(0.025,0.5,0.975))


################################################################
################################################################
####### MuSSE: epiphytic+waterlogged/terrestrial vs. spore type

##data spore type and habit b (epiphytes + waterlogged (1), terrestrial (0))
musse_epiwt<-read_excel("traits_final.xlsx")
musse_epiwt<-as.data.frame(musse_epiwt)
musse_epiwt<-musse_epiwt[,c(1,2,4)]
row.names(musse_epiwt)<-musse_epiwt$Species 
musse_epiwt<-na.omit(musse_epiwt)

match<-match.phylo.data(tree,musse_epiwt)
musse_epiwt2<-match$data
treemuses<-match$phy


treeaultra1 <- force.ultrametric(treemuses)

# Prepare data (character vector)
charstates <- paste0(musse_epiwt2$spore_type,musse_epiwt2$habit_b)
charstates <- as.factor(charstates)
charstates <- as.numeric(charstates)
names(charstates) <- musse_epiwt2$Species

#sampling factor
#1:achlorophyllous terr=7742
#2:achlorophyllous epi+wat=1339+23=1362
#3:chlorophyllous terr=231
#4:chlorophyllous epi+swa=1079+33=1112

table(charstates)
#1   2   3   4 
#951 446  46 315 

f<-c(0.1228,0.327,0.1982759,0.28)
# musse model
liksf <- make.musse(treeaultra1,charstates, k=4,sampling.f = f)

# likelihood function
lik.basesf<-constrain(liksf, mu2 ~ mu1, mu3 ~ mu1, mu4 ~ mu1,
                      q14 ~ 0, q41 ~ 0, q23 ~ 0, q32 ~ 0)
p<-starting.point.musse(treeaultra1,k=4)

fit.basesf <- find.mle(lik.basesf, p[argnames(lik.basesf)],method = "optim",
                       control=list(optim.method="BFGS"))
fit.basesf$convergence
fit.basesf$par

#> 2.323149e-01
#[1] 0.2323149
#0.4646298

#######
#mcmc
priorsf <- make.prior.exponential(0.47)
prelimsf <- diversitree::mcmc(lik.basesf, fit.basesf$par, nsteps=100, prior=priorsf, 
                              w=1, lower=0, print.every=0)

summary(prelimsf)

wsf <- diff(sapply(prelimsf[2:(ncol(prelimsf)-1)], quantile, c(0.05, 0.95)))


#mcmc
samplesew1 <- diversitree::mcmc(lik.basesf, fit.basesf$par, nsteps=10000, w=wsf, lower=0, 
                                  prior=priorsf, print.every=100, 
                                  save.file="mcmc_chain_episwsf_1.csv", save.every=5)

samplesew2 <- diversitree::mcmc(lik.basesf, fit.basesf$par, nsteps=10000, w=wsf, lower=0, 
                                  prior=priorsf, print.every=100, 
                                  save.file="mcmc_chain_episwsf_2.csv", save.every=5)

samplesew3 <- diversitree::mcmc(lik.basesf, fit.basesf$par, nsteps=10000, w=wsf, lower=0, 
                                  prior=priorsf, print.every=100, 
                                  save.file="mcmc_chain_episwsf_3.csv", save.every=5)

samplesew4 <- diversitree::mcmc(lik.basesf, fit.basesf$par, nsteps=10000, w=wsf, lower=0, 
                                  prior=priorsf, print.every=100, 
                                  save.file="mcmc_chain_episwsf_4.csv", save.every=5)

# Analyze runs
library(coda)

runew1 <- coda::mcmc(samplesew1[,-1], start=1, thin=1)
runew2 <- coda::mcmc(samplesew2[,-1], start=1, thin=1)
runew3 <- coda::mcmc(samplesew3[,-1], start=1, thin=1)
runew4 <- coda::mcmc(samplesew4[,-1], start=1, thin=1)

runs_ew <- coda::mcmc.list(runew3,runew4)

par(mar=rep(2,4))
plot(runs_ew)
effectiveSize(runs_ew)
View(runs_ew)
gelman.plot(runs_ew)
gelman.diag(runs_ew)

####posterior probabilities
es1<-read.csv("mcmc_chain_episw1___.csv")
es2<-read.csv("mcmc_chain_episw2___.csv")
es3<-read.csv("mcmc_chain_episw3___.csv")
es4<-read.csv("mcmc_chain_episw4___.csv")

#burn in
bi_es1<-subset(es1, i > 1000)
bi_es2<-subset(es2, i > 1000)
bi_es3<-subset(es3, i > 1000)
bi_es4<-subset(es4, i > 1000)

#merge
ppes<-rbind(bi_es1,bi_es2,bi_es3,bi_es4)
summary(ppes)

#Posterior probabilities transitions rates
mean(ppes$q24 > ppes$q42)
#0.9990278

# 95% HPD rates 
quantile(ppes$mu1,probs=c(0.025,0.5,0.975))





